package com.xy.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.BatchDeleteDto;
import com.xy.dto.CommDto;
import com.xy.dto.SysRoleDto;
import com.xy.entity.SysDeptRole;
import com.xy.entity.SysRole;
import com.xy.entity.SysRoleMenu;
import com.xy.entity.SysUserRole;
import com.xy.mapper.SysRoleMapper;
import com.xy.redis.RedisCache;
import com.xy.service.SysDeptRoleService;
import com.xy.service.SysRoleMenuService;
import com.xy.service.SysRoleService;
import com.xy.service.SysUserRoleService;
import com.xy.utils.MybatisPlusQuery;
import com.xy.utils.PageBean;
import com.xy.utils.R;
import com.xy.vo.SysRoleVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/***
 * 角色服务实现类
 * @author 谭斌
 * @date 2022/12/12 17:17
 */
@Api(tags = "角色管理")
@Service
@RequiredArgsConstructor
public class SysRoleServiceImpl extends ServiceImpl<SysRoleMapper, SysRole> implements SysRoleService {

    private final SysRoleMenuService sysRoleMenuService;
    private final SysDeptRoleService sysDeptRoleService;
    private final SysUserRoleService sysUserRoleService;

    @ApiOperation(value = "新增、更新角色信息", notes = "新增、更新角色信息")
    @PostMapping("saveOrUpdate")
    public R<Long> saveOrUpdate(@Valid @RequestBody SysRoleDto.SaveOrUpdate sysRoleDto) {
        Long id = sysRoleDto.getId();
        LambdaQueryWrapper<SysRole> lqw = new LambdaQueryWrapper<SysRole>()
                .eq(SysRole::getCode, sysRoleDto.getCode())
                .eq(SysRole::getSysId, sysRoleDto.getSysId());
        if (id != null) {
            lqw.ne(SysRole::getId, id);
        }
        long count = count(lqw);
        if (count > 0) {
            return R.fail("此角色标识已存在");
        }
        SysRole sysRole = copy(SysRole.class, sysRoleDto).saveOrUpdate(RedisCache.getLoginId(Long.class), sysRoleDto.getSysId());
        saveOrUpdate(sysRole);
        return R.ok(sysRole.getId());
    }

    @ApiOperation(value = "删除角色", notes = "删除角色")
    @DeleteMapping("batchDelete")
    public R<Boolean> batchDelete(@Valid @RequestBody BatchDeleteDto batchDeleteDto) {
        List<Long> ids = batchDeleteDto.getIds();
        if (CollUtil.isNotEmpty(ids)) {
            for (Long id : ids) {
                deleteRoleById(new CommDto().setId(id));
            }
        }
        return R.ok(Boolean.TRUE);
    }

    @ApiOperation(value = "删除角色", notes = "删除角色")
    @DeleteMapping("deleteRoleById")
    @Transactional(rollbackFor = Exception.class)
    public R<Boolean> deleteRoleById(CommDto commDto) {
        Long id = commDto.getId();
        //角色 sys_role
        removeById(id);
        //角色菜单 sys_role_menu
        sysRoleMenuService.remove(Wrappers.<SysRoleMenu>lambdaQuery().eq(SysRoleMenu::getRoleId, id));
        //部门角色 sys_dept_role
        sysDeptRoleService.remove(Wrappers.<SysDeptRole>lambdaQuery().eq(SysDeptRole::getRoleId, id));
        //用户角色 sys_user_role
        sysUserRoleService.remove(Wrappers.<SysUserRole>lambdaQuery().eq(SysUserRole::getRoleId, id));
        return R.ok(Boolean.TRUE);
    }

    @ApiOperation(value = "获取角色详情", notes = "获取角色详情")
    @GetMapping("{id}")
    public R<SysRoleVo> getById(@PathVariable("id") Long id) {
        SysRole role = super.getById(id);
        SysRoleVo sysRoleVo = copy(SysRoleVo.class, role);
        //角色菜单
        sysRoleVo.setMenuIds(sysRoleMenuService.getRoleMenus(id));
        return R.ok(sysRoleVo);
    }

    @ApiOperation(value = "分页获取角色信息", notes = "分页获取角色信息")
    @PostMapping("page")
    public R<PageBean<SysRoleVo>> getRolePage(@RequestBody SysRoleDto.Page sysRoleDto) {
        PageBean pageBean = sysRoleDto.getPage();
        LambdaQueryWrapper<SysRole> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(sysRoleDto, SysRole.class)
                .like(SysRole::getName)
                .like(SysRole::getDescription)
                .build()
                .orderByDesc(pageBean.getOrders().size() == 0, SysRole::getCreateTime);
        boolean b = RedisCache.getDataAuth(sysRoleDto.getSysId());
        //管理员查询权限查询所有，普通查询权限查询预留和自创建
        if (!b) {
            lambdaQueryWrapper.and(sysRoleLambdaQueryWrapper -> sysRoleLambdaQueryWrapper
                    .eq(SysRole::getCreateTime, RedisCache.getLoginId(Long.class))
                    .or()
                    .eq(SysRole::getIsLeave, true)
            );
        }
        IPage<SysRole> page = page(toIPage(pageBean), lambdaQueryWrapper);
        return R.ok(toPageBean(SysRoleVo.class, page));
    }
}
